#!/usr/bin/perl

# copyright Mark Claypool
# Last significantly modified: April 2008

#
# Compute average intra-coded block size as a measure
# of scene complexity.
#
# Quick usage:
# 1) mpeg_stat -block_info FILE.mpg > FILE.info
# 2) ibs.pl -i FILE.info
#
# Note, all intra mode blocks (I, P and B) are used.
#

$VERSION = 2.2;

require 'getopts.pl';

&ParseCommandLine;

# See if the input file is there and readable
open FP, "< $infile" or die $!;

# initialize count variables
$count = 0;
$size = 0;
$total_count = 0;
$total_size = 0;
$frame_count = 0;

# loop until file done
$line = <FP>;
while ($line) {

  # new frame
  if ($line =~ /^frame/) {
      if ($rate || $per_frame) {
	if ($count != 0) {
          if ($per_frame) {
             printf("%d\t", ($frame_count-1));
          }
	  if ($rate) {
             printf("%.3f\t", ($frame_count-1) / $rate);
          }
        printf ("%.2f\n", $size / $count);
        $size = 0;
        $count = 0;
      }
    }
    $frame_count++;
  }

  # look for intra, then grab size
  if (($line =~ /^block/) && ($line =~ /intra/)) {
    @col = split('\s+', $line);	# split columns on whitespace
    $size += @col[4]; 
    $total_size += @col[4]; 
    $count++;
    $total_count++;
  }

  $line = <FP>;
}

close FP;

# compute and print stats
if ($total_count == 0) {
  printf("No blocks found in \"$infile\".\n");
  printf("Was the file generated with \"mpeg_stat -block_info\"?\n");
  exit(0);
}

$ibs = $total_size / $total_count;

printf("%15s total %6d, ibs %6.3f\n", $infile, $total_count, $ibs);

exit(0);

#######################################################################
# ParseCommandLine
# check for the right number of command line arguments
# print usage message and quit if there is an error
sub ParseCommandLine   {

  &Getopts('hr:i:f');
  if ($opt_h) {
    &usage;
  }
  if ($opt_i) {
    $infile = $opt_i;
  } else {
    printf STDERR "Need to specify input file.\n";
    &usage;
  }
  if ($opt_f) {
    $per_frame = 1;
  } else {
    $per_frame = 0;
  }
  if ($opt_r) {
    $rate = $opt_r;
  } else {
    $rate = 0;
  }
}		

##########################################################################
# usage
# print a usage maessage and quit
sub usage {
  print STDERR "ibs.pl (v$VERSION): average Intra macro Block Size (IBS)\n";
  print STDERR "Usage: ibs.pl <flags>, where flags are:\n";
  print STDERR "\t{-i file}\tinput file (made by mpeg_play -block_info)\n"; 
  print STDERR "\t[-r #]\tframe rate (default off)\n"; 
  print STDERR "\t[-f]\tper frame information (default off)\n"; 
  print STDERR "\t[-h]\tthis usage message\n"; 
  exit;
}
