#!/usr/bin/perl

# copyright Mark Claypool
# Last significantly modified: April 2008

#
# Compute fraction (Percentage) of Forward/backward
# Intra coded Macroblocks (PFIM).
#
# Quick usage:
# 1) mpeg_stat -block_info FILE.mpg > FILE.info
# 2) pfim.pl -i FILE.info
#
# Note, only P and B frames are used.
#

$VERSION = 2.1;

require 'getopts.pl';

&ParseCommandLine;

# See if the input file is there and readable
open FP, "< $infile" or die $!;

$count = 0;
$motion_count = 0;
$total_count = 0;
$total_motion_count = 0;
$frame_count = 0;
$line = <FP>;
while ($line) {

  # new frame
  if ($line =~ /^frame/) {
      if ($rate || $per_frame) {
	if ($count != 0) {
          if ($per_frame) {
             printf("%d\t", ($frame_count-1));
          }
	  if ($rate) {
             printf("%.3f\t", ($frame_count-1) / $rate);
          }
        printf ("%.2f\n", $motion_count / $count);
        $motion_count= 0;
        $count = 0;
      }
    }
    $frame_count++;
  }

  # look for forward, backward or intra
  if (($line =~ /^block/) && (($line =~ /B/) || ($line =~ /P/))) {
    $count++;
    $total_count++;
    if (($line =~ /forw/) || ($line =~ /back/) || ($line =~ /intra/)) {
      $motion_count++;
      $total_motion_count++;
    }
  }

  $line = <FP>;
}

close FP;

# compute and print stats
if ($total_count == 0) {
  printf("No blocks found in \"$infile\".\n");
  printf("Was the file generated with \"mpeg_stat -block_info\"?\n");
  exit(0);
}
$pfim = $total_motion_count/$total_count;

printf("%30s total %8d, motion %8d, pfim %6.3f\n", 
       $infile,  $total_count,  $total_motion_count, $pfim);

exit(0);

#######################################################################
# ParseCommandLine
# check for the right number of command line arguments
# print usage message and quit if there is an error
sub ParseCommandLine   {

  &Getopts('hr:i:f');
  if ($opt_h) {
    &usage;
  }
  if ($opt_i) {
    $infile = $opt_i;
  } else {
    printf STDERR "Need to specify input file.\n";
    &usage;
  }
  if ($opt_f) {
    $per_frame = 1;
  } else {
    $per_frame = 0;
  }
  if ($opt_r) {
    $rate = $opt_r;
  } else {
    $rate = 0;
  }

}		

##########################################################################
# usage
# print a usage maessage and quit
sub usage {
    print STDERR "pfim.pl (v$VERSION): percent of forward or intra-coded macroblocks\n";
    print STDERR "Usage: pfim.pl <flags>, where flags are:\n";
    print STDERR "       {-i file}\tinput file (made by mpeg_play -block_info)\n"; 
    print STDERR "       [-r #]\tframe rate (default off)\n"; 
    print STDERR "       [-f]\tper frame information (default off)\n"; 
    print STDERR "       [-h]\tthis usage message\n"; 
    exit;
}
